<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>VmShepherd - status</title>
    <link href="{{ static('css/bootstrap.min.css') }}" rel="stylesheet">
    <link href="{{ static('css/bootstrap-theme.css') }}" rel="stylesheet">
    <link href="{{ static('css/dashboard.css') }}" rel="stylesheet">
  </head>

  <body>
    <nav class="navbar navbar-inverse navbar-fixed-top">
      <div class="container-fluid">
        <div class="navbar-header">
          <a class="navbar-brand" href="#top">VmShepherd</a>
        </div>
      </div>
    </nav>

    <div class="container-fluid">
      <div class="row">
        <div class="col-sm-3 col-md-2 sidebar">
          <ul class="nav nav-sidebar">
{% for preset in presets|sorted %}
            <li><a href="#{{ preset }}">{{ preset }}</a></li>
{% endfor %}
          </ul>
        </div>
        <div class="col-sm-9 col-sm-offset-3 col-md-10 col-md-offset-2 main">
          <h1 class="page-header">VmShepherd</h1>
    {% include 'preset.html.jinja2' %}
        </div>
      </div>
    </div>

    <!-- Bootstrap core JavaScript
    ================================================== -->
    <!-- Placed at the end of the document so the pages load faster -->
    <script src="{{ static('js/jquery.min.js') }}"></script>
    <script src="{{ static('js/bootstrap.min.js') }}"></script>
  </body>
</html>

